/*
  cpld_isa.c - Tests ISA (PC/104) card on CPLD
              by Micromint Support <support@micromint.com>    Jan-2009
*/

#include "inc/hw_gpio.h"
#include "inc/hw_memmap.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/systick.h"
#include "driverlib/ssi.h"
#include "driverlib/sysctl.h"
#include "utils/uartstdio.h"
#include "xio.h"

#define LED_PIN                 GPIO_PIN_1
#define LED_REG                 HWREG(GPIO_PORTE_BASE + (GPIO_O_DATA + (LED_PIN << 2)))

/*
  ! \addtogroup example_list
  ! <h1>CPLD ISA card(cpld_isa)</h1>
  !
  ! Tests ISA card on CPLD.
*/

/*
  Error routine called on driver library errors
*/
#ifdef DEBUG
void __error__(char *pcFilename, unsigned long ulLine)
{
}
#endif

/*
  Handler for SysTick interrupt
*/
void SysTickHandler(void)
{
    static unsigned long n;
    unsigned long i;
    unsigned long digit[] = {0x3f, 0x06, 0x5b, 0x4f, 0x66,
                             0x6d, 0x7d, 0x07, 0x7f, 0x6f};

    /* Handle event */
    n %= 10;
    LED_REG ^= LED_PIN; /* Toggle LED */

    IsaWrite(0x0106, digit[n]); /* Send digits to SSD on I/O card at 0x0106 */
    i = IsaRead(0x0107); /* Read dip switch on I/O card at 0x0107 */
    UARTprintf("ISA Read=%d ISA Write=%d\n",
				i, digit[n]); /* UART message */

    n++;
}

/*
  Main function to test ISA card on CPLD
*/
int main(void)
{
    /* If running on Rev A2 silicon, turn the LDO voltage up to 2.75V */
    if(REVISION_IS_A2)
    {
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }

    /* Set the processor to run at 50 MHz */
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);

    /* Enable peripherals and set GPIO pins */
    /* --- Initialize ISA controller --- */
    IsaInit();
    /* --- Enable LED --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    //GPIOPinTypeGPIOOutput(GPIO_PORTE_BASE, LED_PIN);
    HWREG(GPIO_PORTE_BASE + GPIO_O_DIR) |= LED_PIN;
    HWREG(GPIO_PORTE_BASE + GPIO_O_DEN) |= LED_PIN;
    /* --- Enable UART --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    //SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);

    /* Configure SysTick to interrupt once per second */
    SysTickPeriodSet(SysCtlClockGet());
    SysTickEnable();
    SysTickIntEnable();

    /* Test ISA card */
    UARTprintf("CPLD ISA\n");
    /* Loop while waiting for interrupts */
    while(1) {}
}
