/*
  cpld_xio.c - Tests extended I/O and LCD interfaces on CPLD
              by Micromint Support <support@micromint.com>    Jan-2009
*/

#include "inc/hw_gpio.h"
#include "inc/hw_memmap.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/systick.h"
#include "driverlib/ssi.h"
#include "driverlib/sysctl.h"
#include "utils/uartstdio.h"
#include "xio.h"
#include "lcd.h"

#define LED_PIN                 GPIO_PIN_1
#define LED_REG                 HWREG(GPIO_PORTE_BASE + (GPIO_O_DATA + (LED_PIN << 2)))

/*
  ! \addtogroup example_list
  ! <h1>Extended I/O(cpld_xio)</h1>
  !
  ! Test extended I/O and LCD interfaces on CPLD
*/

/*
  Error routine called on driver library errors
*/
#ifdef DEBUG
void __error__(char *pcFilename, unsigned long ulLine)
{
}
#endif

/*
  Handler for SysTick interrupt
*/
void SysTickHandler(void)
{
    static unsigned long n;
    unsigned long i;
    unsigned long data[] = {0x01, 0x02, 0x04, 0x08,
                            0x10, 0x20, 0x40, 0x80};
    char *msg[] = {"Testing LCD...      ",
                  "The quick brown fox ",
                  "jumps over the lazy ",
                  "dog. !@#$%^&*()_+,\"\'"};

    /* Handle event */
    n %= 8;
    LED_REG ^= LED_PIN; /* Toggle LED */
    XioWrite(XIO_PXA, data[n]); /* Send data to ports XA and XB */
    XioWrite(XIO_PXB, ~data[n]);
    i = XioRead(XIO_PXC); /* Read data from port XC */
    UARTprintf("LED=%d PXA=%d PXC=%d\n", LED_REG, data[n], i); /* UART message */

    for (i = 0; i<4; i++)
        LcdWriteAt(0, i, msg[(n + i) % 4]); // Send strings to LCD

    n++;
}

/*
  Main function to extended I/O and LCD interfaces on CPLD
*/
int main(void)
{
    /* If running on Rev A2 silicon, turn the LDO voltage up to 2.75V */
    if(REVISION_IS_A2)
    {
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }

    /* Set the processor to run at 50 MHz */
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);

    /* Enable peripherals and set GPIO pins */
    /* --- Initialize LCD controller --- */
    LcdInit();
    // --- Enable LED ---
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    //GPIOPinTypeGPIOOutput(GPIO_PORTE_BASE, LED_PIN);
    HWREG(GPIO_PORTE_BASE + GPIO_O_DIR) |= LED_PIN;
    HWREG(GPIO_PORTE_BASE + GPIO_O_DEN) |= LED_PIN;
    /* --- Enable UART --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    //SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);

    /* Configure SysTick to interrupt twice per second (500 ms)  */
    SysTickPeriodSet(SysCtlClockGet()/2);
    SysTickEnable();
    SysTickIntEnable();

    /* Test extended I/O's and LCD */
    UARTprintf("CPLD XIO\n");
    /* Loop while waiting for interrupts */
    while(1) {}
}
