--------------------------------------------------------------------------------
-- isa_master.vhd : ISA bus master
-- Micromint USA (407)333-4799 www.micromint.com
-- Micromint Support <support@micromint.com>
-- 
-- Version:        2009-07
-- Design Name:    ISA Master
-- Module Name:    isa_master - Behavioral 
-- Project Name:   Eagle 100
-- Target Devices: xc9572xl
-- Tool versions:  Xilinx ISE 11.1
--
-- Description: 
--   Implements a simple ISA bus master for 8-bit I/O
--
-- Revision history: 
--   2009-07 - 07/31/2009 Upgraded to Xilinx ISE 11.1.
--   2009-01 - 01/27/2009 Refactoring to reduce macrocell usage
--   2008-07 - 07/02/2008 Upgraded to Xilinx ISE 10.1
--   2008-04 - 04/09/2008 Initial build using Xilinx ISE 9.2
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.ALL;
use ieee.std_logic_unsigned.ALL;

entity isa_master is
	port (
		clk : in  std_logic;
		reset : in std_logic;
		AEN : out std_logic;
		BALE : out std_logic;
		IOR : out std_logic;
		IOW : out std_logic;
		enable : in std_logic; -- 0=disable, 1=enable
		RW : in std_logic -- 0=read, 1=write
		);
end isa_master;

architecture Behavioral of isa_master is
	type isaState is (idle, addrSet, addrLatch, setRead, setWrite);
--	type isaState is (idle, addrSet, addrLatch, setRead,
--							dataRead, setWrite, dataWrite);
	signal nState, cState: isaState; -- Next state, current state
	-- Timing constants assuming 16 MHz clock (1 = 62.5 ns)
--	constant tAddrLatch : integer := 1; -- 62.5 ns
--	constant tSetRead : integer := 8; -- 500 ns
--	constant tSetWrite : integer := 2; -- 125 ns
--	constant tMin : integer := 1; -- 62.5 ns
--	constant tMax : integer := 8;
--	signal tState: integer range 0 to tMax;
begin

	AEN <= '1' when cState = idle else
			'0';

	state_process : process(clk, reset, enable)
--		variable count : integer range 0 to tMax;
		variable pEnable : std_logic := '0'; -- Previous enable
	begin
		if (reset = '1') then
			cState <= idle;
--			count := 0;
		elsif rising_edge(clk) then
			if (pEnable = '0' and enable = '1') then
				cState <= addrSet;
--				count := 0;
			else
--				count := count + 1;
--				if (count = tState) then
					cState <= nState;
--					count := 0;
--				end if;
			end if;
			pEnable := enable;
		end if;
	end process state_process;
	 
	control_process : process(cState)
	begin
		case cState is 
			when idle =>
				BALE <= '0'; IOR <= '1'; IOW <= '1';
--				tState <= tMin;
				nState <= idle;
			when addrSet =>
				BALE <= '0'; IOR <= '1'; IOW <= '1';
--				tState <= tMin;
				nState <= addrLatch;
			when addrLatch =>
				BALE <= '1'; IOR <= '1'; IOW <= '1';
--				tState <= tAddrLatch;
				if (RW = '1') then
					nState <= setWrite;
				else
					nState <= setRead;
				end if;
			when setRead =>
				BALE <= '0'; IOR <= '0'; IOW <= '1';
--				tState <= tSetRead;
--				nState <= dataRead;
--			when dataRead =>
--				BALE <= '0'; IOR <= '0'; IOW <= '1';
--				dataRX <= data; -- Load data from slave
--				tState <= tMin;
				nState <= idle;
			when setWrite =>
				BALE <= '0'; IOR <= '1'; IOW <= '0';
--				tState <= tSetWrite;
--				nState <= dataWrite;
--			when dataWrite =>
--				BALE <= '0'; IOR <= '1'; IOW <= '0';
--				data <= dataTX; -- Store data to slave
--				tState <= tMin;
				nState <= idle;
		end case;
	end process control_process;
end Behavioral;
